#!/usr/bin/python
#shuber, 2008-06-04

__author__ = "shuber"


import gtk
import pango
import pty
import string
import time
import threading
import vte

import ClientIOTerminal



class DbgTerminal (vte.Terminal):

	isactive = True
	lastrow = 0
	history = []


	def __init__(self, clientCmd):

		vte.Terminal.__init__(self)

		#Start debugger
		self.clientCmd = clientCmd
		#Open pseudo-terminal where to-be-debugged process reads/writes to
		self.client_ptymaster, self.client_ptyslave = pty.openpty()

		#Set up terminal window and initialize debugger
		self.connect("cursor-moved", self.contents_changed)
		self.connect("child-exited", lambda *w: gtk.main_quit())

		#font description
		fontdesc = pango.FontDescription("monospace 9")
		self.set_font(fontdesc)


	def initialize(self):
		#Launch debugger
		self.fork_command( self.getCommand(), self.getArgv())
		self.setPty(self.client_ptyslave)


	def contents_changed(self, term):
		c,r = term.get_cursor_position()

		if self.lastrow <= r:
			text = self.get_text_range(self.lastrow,0,r,-1,lambda *w:True)

			#Remove the incomplete line
			if self.getHistoryLen()>0 and (len(self.history[-1])==0 or self.history[-1]!='\n') :
				del self.history[-1]

			#Get the lines and remove empty lines
			lines = string.split(text, "\n")

			#Remove last empty line...
			if lines[-1] == "":
				del lines[-1]

			#Add lines to history
			self.history += [l+"\n" for l in lines[:-1]]
			self.history += [lines[-1]]
			self.lastrow = r


	def waitForNewline(self):
		r = self.lastrow
		while not self.lastrow > r:
			gtk.main_iteration()

	def getHistoryLen(self):
		return len(self.history)

	def waitForRx(self, rx, start=None):	

		if start == None:
			start = self.getHistoryLen()
		if start < 0:
			start = 0

		while True:
			for no in range(start, self.getHistoryLen()):
				line = self.history[no]
				if rx.search(line):
					return no, line

			start = self.getHistoryLen()
			gtk.main_iteration()


	def getCommand(self):
		return self.getArgv()[0];

	def getArgv(self):
		raise NotImplementedError()

	def setPty(self, pty):
		raise NotImplementedError()

	def setRun(self):
		raise NotImplementedError()

	def setContinue(self):
		raise NotImplementedError()

	def setStepover(self):
		raise NotImplementedError()

	def setStepin(self):
		raise NotImplementedError()

	def setQuit(self):
		raise NotImplementedError()

	def setBreakpoint(self, file, lineno, condition=False):
		raise NotImplementedError()

	def delBreakpoint(self, breakpoint):
		raise NotImplementedError()

	def getExpression(self, expr):
		raise NotImplementedError()

	def waitForActivation(self, his):		
		raise NotImplementedError()

	def setActive(self, isactive):
		self.isactive = isactive

	def isActive(self):
		return self.isactive

	
	



class DbgWindow (gtk.Window):

	clientIOWnd = None


	def __init__(self, terminal):

		#Set up some members
		self.terminal = terminal

		#Set up GTK stuff
		gtk.Window.__init__(self)
		self.connect("destroy", lambda *w: gtk.main_quit())

		#Set title and add terminal
		self.set_title("Debugger I/O")
		self.terminal.history = []
		self.terminal.history_length = 5
		self.add(self.terminal)

		#Show the window
		self.show_all()

	def toggleClientIOWindow(self):
		if not self.clientIOWnd:
			self.clientIOWnd = ClientIOTerminal.ClientIOWindow(self, \
					self.terminal.client_ptymaster)
		else:
			self.clientIOWnd.destroy()
			self.clientIOWnd = None

	def isClientIOWindowExisting(self):
		return self.clientIOWnd != None



